# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

from pathlib import Path
from typing import Any, Optional, Union

import ruamel.yaml

from . import models as cici_config
from .converter import CONVERTER


def loads(
    text: str,
    gitlab_ci_jobs: Optional[dict[str, Any]] = None,
    precommit_hooks: Optional[dict[str, Any]] = None,
) -> cici_config.File:
    if gitlab_ci_jobs is None:
        gitlab_ci_jobs = {}
    if precommit_hooks is None:
        precommit_hooks = {}
    yaml = ruamel.yaml.YAML(typ="safe")
    data = yaml.load(text)
    data.setdefault("targets", [])
    for target in data["targets"]:
        if target["name"] in precommit_hooks:
            target["precommit_hook"] = {"name": target["name"]}
        if target["name"] in gitlab_ci_jobs:
            target["gitlab_include"] = {"name": target["name"]}
    return CONVERTER.structure(data, cici_config.File)


def load(
    file: Union[str, Path],
    gitlab_ci_jobs: Optional[dict[str, Any]] = None,
    precommit_hooks: Optional[dict[str, Any]] = None,
) -> cici_config.File:
    return loads(
        open(file).read(),
        gitlab_ci_jobs=gitlab_ci_jobs,
        precommit_hooks=precommit_hooks,
    )



# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

import functools

import cattrs
from cattrs.gen import make_dict_structure_fn, override

from . import models

CONVERTER = cattrs.Converter(omit_if_default=True, forbid_extra_keys=True)


make_dict_struct = functools.partial(
    make_dict_structure_fn,
    _cattrs_forbid_extra_keys=True,
)


def structure_variables(object, __):
    variables = {}
    for key, value in object.items():
        value["name"] = key
        variables[key] = CONVERTER.structure(value, models.Variable)
    return variables


CONVERTER.register_structure_hook(
    models.File,
    make_dict_struct(
        models.File,
        CONVERTER,
        variables=override(struct_hook=structure_variables),
    ),
)



# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

from typing import Optional

from attrs import define, field


@define(frozen=True, kw_only=True, slots=True)
class PreCommitHookTarget:
    name: str


@define(frozen=True, kw_only=True, slots=True)
class GitLabIncludeTarget:
    name: str


@define(frozen=True, kw_only=True, slots=True)
class Group:
    name: str
    brief: str = ""
    description: str = ""


@define(frozen=True, kw_only=True, slots=True)
class Target:
    name: str
    brief: str = ""
    description: str = ""

    groups: list[str] = field(factory=list)

    precommit_hook: Optional[PreCommitHookTarget] = None
    gitlab_include: Optional[GitLabIncludeTarget] = None


@define(frozen=True, kw_only=True, slots=True)
class VariableExample:
    value: str
    brief: str = ""


@define(frozen=True, kw_only=True, slots=True)
class Variable:
    name: str
    brief: str = ""
    default: str = ""
    description: str = ""
    required: bool = False

    examples: list[VariableExample] = field(factory=list)


@define(kw_only=True, slots=True)
class File:
    name: str

    repo_url: str = ""

    gitlab_project_path: str = ""

    brief: str = ""
    description: str = ""

    groups: list[Group] = field(factory=list)
    targets: list[Target] = field(factory=list)
    variables: dict[str, Variable] = field(factory=dict)
